#include "pacman.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <stack>
#include <set>
#include <map>

typedef long long llong;
const int MAXN = 500000 + 10;
const int MAXLOG = 20;
const llong INF = 1e18;
const int INTINF = 1e9;

int n, m;
int w[MAXN];
int s[MAXN];

struct Sparse
{
    int sparse[MAXLOG][MAXN];
    int getLOG[MAXN];

    int cmp(int left, int right)
    {
        if (w[left] > w[right]) return left;
        return right;
    }

    void build()
    {
        for (int i = 1 ; i <= n ; ++i)
        {
            sparse[0][i] = i;
        }

        for (int log = 1 ; (1 << log) <= n ; ++log)
        {
            for (int i = 1 ; i + (1 << log) - 1 <= n ; ++i)
            {
                sparse[log][i] = cmp(sparse[log - 1][i], sparse[log - 1][i + (1 << log - 1)]);
            }
        }

        for (int len = 1 ; len <= n ; ++len)
        {
            getLOG[len] = getLOG[len - 1];
            if ((1 << getLOG[len] + 1) < len) getLOG[len]++;
        }
    }

    int findMAX(int l, int r)
    {
        int log = getLOG[r - l + 1];
        return cmp(sparse[log][l], sparse[log][r - (1 << log) + 1]);
    }
};

int prev[MAXN];
int next[MAXN];
std::map <std::pair <int,int>, int> mem;
std::stack <int> st;
Sparse sparse;

llong f(int l, int r)
{
    if (l > r) return 0;
    if (mem.count({l, r}))
    {
        return mem[{l, r}];
    }

    int max = sparse.findMAX(l, r);
    return mem[{l, r}] = std::max(f(l, max - 1), f(max + 1, r)) + 1;
}

llong solve(const std::vector <int> &W, const std::vector <int> &S)
{
    n = W.size();
    m = S.size();

    for (int i = 1 ; i <= n ; ++i)
    {
        w[i] = W[i - 1];
    }

    for (int i = 1 ; i <= m ; ++i)
    {
        s[i] = S[i - 1];
    }

    std::sort(s + 1, s + 1 + m);
    sparse.build();
    llong ans = 0;

    for (int l = 1 ; l <= m ; ++l)
    {
        for (int r = l ; r <= m ; ++r)
        {
            ans += f(s[l], s[r]);
        }
    }

    return ans;
}

