#include "pacman.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <stack>
#include <set>
#include <map>

#pragma attribute ((__always_inline__)) inline
#pragma GCC optimize ("unroll_loops")
#pragma GCC target ("sse4")

typedef long long llong;
const int MAXN = 500000 + 10;
const llong INF = 1e18;
const int INTINF = 1e9;

int n, m;
int w[MAXN];
int s[MAXN];
int prev[MAXN];
int next[MAXN];
int left[MAXN];
int right[MAXN];
int depth[MAXN];
int nextDepth[MAXN];
std::stack <int> st;
int len;

void dfs(int node)
{
    if (node == 0)
    {
        return;
    }

    depth[left[node]] = depth[node] + 1;
    depth[right[node]] = depth[node] + 1;
    dfs(left[node]);
    dfs(right[node]);
}

llong query(int l, int r)
{
    int ans = 0;
    int idx = l;
    while (nextDepth[idx] <= r)
    {
        idx = nextDepth[idx];
    }

    int curr = -1;
    int save = idx;

    while (idx >= l)
    {
        curr++;
        idx = prev[idx];
    }

    idx = save;
    while (idx <= r)
    {
        curr++;
        idx = next[idx];
    }

    ans = std::max(ans, curr);
    idx = save;

    return ans;
}

llong solve(const std::vector <int> &W, const std::vector <int> &S)
{
    n = W.size();
    m = S.size();

    for (int i = 1 ; i <= n ; ++i)
    {
        w[i] = W[i - 1];
    }

    for (int i = 1 ; i <= m ; ++i)
    {
        s[i] = n + 1 - S[i - 1];
    }

    std::sort(s + 1, s + 1 + m);
    std::reverse(w + 1, w + 1 + n);
    int max = -1;

    for (int r = 1 ; r <= n ; ++r)
    {
        if (w[r] == n)
        {
            max = r;
        }

        int lastPopped = 0;
        while (st.size() && w[st.top()] < w[r])
        {
            lastPopped = st.top();
            st.pop();
        }

        left[r] = lastPopped;
        if (st.size()) right[st.top()] = r;
        st.push(r);
    }

    dfs(max);
    depth[n + 1] = n + 1;
    while (st.size()) st.pop();
    st.push(n + 1);

    for (int i = n ; i >= 1 ; --i)
    {
        while (depth[st.top()] <= depth[i])
        {
            st.pop();
        }

        nextDepth[i] = st.top();
        st.push(i);
    }

    while (st.size()) st.pop();
    w[0] = w[n + 1] = n + 1;
    st.push(0);

    for (int i = 1 ; i <= n ; ++i)
    {
        while (w[st.top()] < w[i])
        {
            st.pop();
        }

        prev[i] = st.top();
        st.push(i);
    }

    while (st.size())
    {
        st.pop();
    }

    st.push(n + 1);
    for (int i = n ; i >= 1 ; --i)
    {
        while (w[st.top()] < w[i])
        {
            st.pop();
        }

        next[i] = st.top();
        st.push(i);
    }

    llong ans = 0;
    for (int l = 1 ; l <= m ; ++l)
    {
        for (int r = l ; r <= m ; ++r)
        {
            ans += query(s[l], s[r]);
        }
    }

    return ans;
}
