#include "pacman.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <stack>
#include <set>
#include <map>

/**
#pragma GCC optimize ("unroll_loops")
#pragma GCC target ("sse4")
*/

typedef long long llong;
const int MAXN = 500000 + 10;
const int MAXM = 1000 + 10;
const llong INF = 1e18;
const int INTINF = 1e9;

int n, m;
int w[MAXN];
int s[MAXN];
struct Cartesian
{
    struct Node
    {
        Node *left;
        Node *right;
        Node *parent;
        int mySize;
        int idx;

        Node()
        {
            mySize = 0;
            left = right = parent = nullptr;
        }

        Node(int _idx)
        {
            idx = _idx;
            mySize = 1;
            left = right = parent = nullptr;
        }
    };

    Node *root;
    Node *lastRight;
    void reset()
    {
        root = lastRight = nullptr;
    }

    void recover(Node *node)
    {
        if (node == nullptr)
        {
            return;
        }

        node->mySize = 1;
        if (node->left != nullptr)
        {
            node->mySize = std::max(node->mySize, node->left->mySize + 1);
        }

        if (node->right != nullptr)
        {
            node->mySize = std::max(node->mySize, node->right->mySize + 1);
        }
    }

    void merge(Node *&node, Node *left, Node *right)
    {
        if (left == nullptr)
        {
            node = right;
            return;
        }

        if (right == nullptr)
        {
            node = left;
            return;
        }

        if (w[left->idx] > w[right->idx])
        {
            node = left;
            merge(node->right, left->right, right);
        } else
        {
            node = right;
            merge(node->left, left, right->left);
        }

        recover(node);
    }

    void push(int idx)
    {
        Node *newNode = new Node(idx);
        Node *ptr = lastRight;
        Node *last = nullptr;

        while (ptr != nullptr && w[ptr->idx] < w[idx])
        {
            last = ptr;
            ptr = ptr->parent;
        }

        newNode->left = last;
        if (ptr != nullptr) ptr->right = newNode;
        else root = newNode;
        lastRight = newNode;
    }

    void fix(Node *curr)
    {
        if (curr == nullptr)
        {
            return;
        }

        fix(curr->left);
        fix(curr->right);
        recover(curr);
    }

    void fix()
    {
        fix(root);
    }

    void pushRight(int idx)
    {
        Node* newNode = new Node(idx);
        merge(root, root, newNode);
    }

    void pushLeft(int idx)
    {
        Node* newNode = new Node(idx);
        merge(root, newNode, root);
    }

    void mergeWith(Cartesian &curr)
    {
        merge(root, root, curr.root);
    }

    int getAnswer()
    {
        return root->mySize;
    }
};

int prev[MAXN];
int next[MAXN];
int left[MAXN];
int right[MAXN];
int depth[MAXN];
int nextDepth[MAXN];
std::stack <int> st;
Cartesian cartesian[MAXM];
int len;

void dfs(int node)
{
    if (node == 0)
    {
        return;
    }

    depth[left[node]] = depth[node] + 1;
    depth[right[node]] = depth[node] + 1;
    dfs(left[node]);
    dfs(right[node]);
}

int calc(int l, int r, int idx)
{
    int curr = -1;
    int save = idx;
    while (idx >= l)
    {
        curr++;
        idx = prev[idx];
    }

    idx = save;
    while (idx <= r)
    {
        curr++;
        idx = next[idx];
    }

    return curr;
}

llong query(int l, int r)
{
    int ans = 0;
    int idx = l;
    while (nextDepth[idx] <= r && calc(l, r, nextDepth[idx]) >= calc(l, r, idx))
    {
        idx = nextDepth[idx];
    }

    return calc(l, r, idx);
}

llong query2(int l, int r)
{
    int ans = 0;
    int idx = l;
    while (next[idx] <= r && calc(l, r, next[idx]) >= calc(l, r, idx))
    {
        idx = next[idx];
    }

    return calc(l, r, idx);
}

llong solve(const std::vector <int> &W, const std::vector <int> &S)
{
    n = W.size();
    m = S.size();

    for (int i = 1 ; i <= n ; ++i)
    {
        w[i] = W[i - 1];
    }

    for (int i = 1 ; i <= m ; ++i)
    {
        s[i] = S[i - 1];
    }

    std::sort(s + 1, s + 1 + m);
    for (int idxL = 1 ; idxL < m ; ++idxL)
    {
        cartesian[idxL].reset();
        for (int idx = s[idxL] + 1 ; idx < s[idxL + 1] ; ++idx)
        {
            cartesian[idxL].push(idx);
        }
    }

    for (int i = 1 ; i < m ; ++i)
    {
        cartesian[i].fix();
    }

    /*std::cout << "before merging\n";
    for (int i = 1 ; i < m ; ++i)
    {
        cartesian[i].printTree();
    }*/


    llong ans = m;
    for (int idxL = 1 ; idxL <= m ; ++idxL)
    {
        cartesian[idxL].pushLeft(s[idxL]);
        for (int idxR = idxL + 1 ; idxR <= m ; ++idxR)
        {
            cartesian[idxL].pushRight(s[idxR]);
            ans += cartesian[idxL].getAnswer();
            if (idxR < m) cartesian[idxL].mergeWith(cartesian[idxR]);
            /*{
                // std::cout << "after merging: " << idxL << ' ' << idxR << '\n';
                cartesian[idxL].printTree();
            }*/
        }
    }

    return ans;
}
