#include "pacman.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <stack>
#include <set>
#include <map>

#pragma attribute ((__always_inline__)) inline
#pragma GCC optimize ("unroll_loops")
#pragma GCC target ("sse4")

typedef long long llong;
const int MAXN = 500000 + 10;
const llong INF = 1e18;
const int INTINF = 1e9;

int n, m;
int w[MAXN];
int s[MAXN];
struct Cartesian
{
    struct Node
    {
        int idx;
        int leftDepth;
    };

    int maxDepth;
    std::vector <Node> rightChain;

    Cartesian()
    {
        maxDepth = 0;
    }

    void reset()
    {
        maxDepth = 0;
        rightChain.clear();
    }

    void push(int idx)
    {
        int currMAX = -1;
        while (rightChain.size() && w[rightChain.back().idx] < w[idx])
        {
            currMAX = std::max(currMAX + 1, rightChain.back().leftDepth);
            rightChain.pop_back();
        }

        rightChain.push_back({idx, currMAX + 1});
        maxDepth = std::max(maxDepth, (int)rightChain.back().leftDepth + (int)rightChain.size());
    }

    int getAnswer()
    {
        return maxDepth;
    }
};

int prev[MAXN];
int next[MAXN];
int left[MAXN];
int right[MAXN];
int depth[MAXN];
int nextDepth[MAXN];
std::stack <int> st;
Cartesian cartesian;
int len;

void dfs(int node)
{
    if (node == 0)
    {
        return;
    }

    depth[left[node]] = depth[node] + 1;
    depth[right[node]] = depth[node] + 1;
    dfs(left[node]);
    dfs(right[node]);
}

int calc(int l, int r, int idx)
{
    int curr = -1;
    int save = idx;
    while (idx >= l)
    {
        curr++;
        idx = prev[idx];
    }

    idx = save;
    while (idx <= r)
    {
        curr++;
        idx = next[idx];
    }

    return curr;
}

llong query(int l, int r)
{
    int ans = 0;
    int idx = l;
    while (nextDepth[idx] <= r && calc(l, r, nextDepth[idx]) >= calc(l, r, idx))
    {
        idx = nextDepth[idx];
    }

    return calc(l, r, idx);
}

llong query2(int l, int r)
{
    int ans = 0;
    int idx = l;
    while (next[idx] <= r && calc(l, r, next[idx]) >= calc(l, r, idx))
    {
        idx = next[idx];
    }

    return calc(l, r, idx);
}

llong solve(const std::vector <int> &W, const std::vector <int> &S)
{
    n = W.size();
    m = S.size();

    for (int i = 1 ; i <= n ; ++i)
    {
        w[i] = W[i - 1];
    }

    for (int i = 1 ; i <= m ; ++i)
    {
        s[i] = S[i - 1];
    }

    llong ans = 0;
    std::sort(s + 1, s + 1 + m);
    for (int idxL = 1 ; idxL <= m ; ++idxL)
    {
        int ptr = idxL;
        int l = s[idxL];
        cartesian.reset();
        for (int r = l ; r <= s[m] ; ++r)
        {
            cartesian.push(r);
            if (r == s[ptr])
            {
                ans += cartesian.getAnswer();
                ptr++;
            }
        }
    }


    return ans;
}